/*
 * humaxsrc.c -- Humax Serial Remote Control
 * Copyright (C) 2003 Charles Yates <charles.yates@pandora.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* System header files */
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* Application header files */
#include "humaxsrc.h"

/** Sleep for a given number of milliseconds.
*/

static void sleep_ms( int ms )
{
	usleep( ms * 1000 );
}

struct command_mapping
{
	char *command;
	short val;
};

struct command_mapping command_table[] = 
{
	{ "onoff", KEY_ONOFF },
	{ "tvsat", KEY_TVSAT },
	{ "radio", KEY_RADIO },
	{ "menu", KEY_MENU },
	{ "up", KEY_UP },
	{ "left", KEY_LEFT },
	{ "ok", KEY_OK },	
	{ "right", KEY_RIGHT },
	{ "down", KEY_DOWN },
	{ "exit", KEY_EXIT },
	{ "ask", KEY_ASK },
	{ "mute", KEY_MUTE },
	{ "white", KEY_WHITE },
	{ "yellow", KEY_YELLOW },
	{ "epg", KEY_EPG },
	{ "red", KEY_RED },
	{ "green", KEY_GREEN },
	{ "blue", KEY_BLUE },
	{ "vplus", KEY_VPLUS },
	{ "vminus", KEY_VMINUS },
	{ "rcl", KEY_RCL },
	{ "tvradio", KEY_FRONT_TVRADIO },
	{ "standby", KEY_FRONT_STANDBY },
	{ "front_down", KEY_FRONT_DOWN },
	{ "front_up", KEY_FRONT_UP },
	{ "plus", KEY_PPLUS },
	{ "minus", KEY_PMINUS },
	{ "", -1 }
};

struct rc_mapping
{
	char rc;
	short val;
};

struct rc_mapping control_table[] = 
{
	{ '0', KEY_0 },
	{ '1', KEY_1 },
	{ '2', KEY_2 },
	{ '3', KEY_3 },
	{ '4', KEY_4 },
	{ '5', KEY_5 },
	{ '6', KEY_6 },
	{ '7', KEY_7 },
	{ '8', KEY_8 },
	{ '9', KEY_9 },
	{ '+', KEY_PPLUS },
	{ '-', KEY_PMINUS },
	{ '\0', -1 }
};

static short map_command_to_code( char *command )
{
	short val = -1;
	int index = 0;

	for ( index = 0; command_table[ index ].val != -1; index ++ )
	{
		if ( !strcmp( command, command_table[ index ].command ) )
		{
			val = command_table[ index ].val;
			break;
		}
	}

	return val;
}

static short map_char_to_code( char rc )
{
	short val = -1;
	int index = 0;

	for ( index = 0; control_table[ index ].val != -1; index ++ )
	{
		if ( rc == control_table[ index ].rc )
		{
			val = control_table[ index ].val;
			break;
		}
	}

	return val;
}

/** Open the serial device to which the humax is connected.
*/

int humax_init( char *device )
{
	int fd = open( device, O_WRONLY );
	if ( fd != -1 )
	{
		int error = 0;
		struct termios term;

		error = tcgetattr( fd, &term );

		// send IR control command (115200, 8N1)
		if ( error == 0 )	
			error = cfsetospeed( &term, B115200 );
		if ( error == 0 )
			error = tcsetattr( fd, TCSANOW, &term );

		if ( error != 0 )
		{
			close( fd );
			fd = -1;
		}
	}
	return fd;
}

/** Send a command.
*/

void humax_send( int fd, short code )
{
	char cmd[16];
	char *p = cmd;

	sprintf( cmd, "%02hxt%c", code, (char)0xD );

	while( *p )
	{
		write( fd, p, 1 );
		sleep_ms( 3 );
		p ++;
	}

	sleep_ms( 80 );
}

/** Execute a high level command.
*/

int humax_execute( int fd, char *command )
{
	int error = 0;
	short val = -1;
	int pos = 0;

	val = map_command_to_code( command );
	if ( val == -1 )
	{
		for ( pos = 0; pos < strlen( command ); pos ++ )
		{
			short val = map_char_to_code( command[ pos ] );
			if ( val != -1 )
				humax_send( fd, val );
			else
				break;
		}
	}
	else
	{
		humax_send( fd, val );
	}
	
	if ( val == -1 )
		error = -1;
	
	return error;
}

/** Convenience function to obtain each command available.
*/

char *humax_get_command( int index )
{
	char *cmd = NULL;
	if ( command_table[ index ].val != -1 )
		cmd = command_table[ index ].command;
	return cmd;
}

/** Close the device.
*/

void humax_close( int fd )
{
	close( fd );
}
